@@ -1,5 +1,5 @@
 /* Copyright (C) 2003-2004 Douglas Thain and the University of Wisconsin
- * Copyright (C) 2005- The University of Notre Dame
+ * Copyright (C) 2022 The University of Notre Dame
  * This software is distributed under the GNU General Public License.
  * See the file COPYING for details.
 */
@@ -8,7 +8,6 @@
 #include "chirp_alloc.h"
 #include "chirp_audit.h"
 #include "chirp_filesystem.h"
-#include "chirp_fs_confuga.h"
 #include "chirp_group.h"
 #include "chirp_job.h"
 #include "chirp_protocol.h"
@@ -30,9 +29,9 @@
 #include "getopt_aux.h"
 #include "host_disk_info.h"
 #include "host_memory_info.h"
-#include "json.h"
 #include "jx.h"
 #include "jx_print.h"
+#include "jx_parse.h"
 #include "link.h"
 #include "list.h"
 #include "load_average.h"
@@ -245,7 +244,10 @@ static int update_all_catalogs(const char *url)
 
 	char *message = jx_print_string(j);
 
-	list_iterate(catalog_host_list, (list_op_t) catalog_query_send_update, message);
+	const char *host;
+	LIST_ITERATE(catalog_host_list,host) {
+		catalog_query_send_update(host,message,CATALOG_UPDATE_BACKGROUND);
+	}
 
 	free(message);
 	jx_delete(j);
@@ -705,7 +707,7 @@ static void chirp_handler(struct link *l, const char *addr, const char *subject)
 					if(!strncmp(d->name, ".__", 3))
 						continue;
 					chirp_stat_encode(B, &d->info);
-					link_putfstring(l, "%s\n%s\n", stalltime, d->name, buffer_tostring(B));
+					link_printf(l, stalltime, "%s\n%s\n", d->name, buffer_tostring(B));
 					buffer_rewind(B, 0);
 				}
 				cfs->closedir(dir);
@@ -727,7 +729,7 @@ static void chirp_handler(struct link *l, const char *addr, const char *subject)
 				while((d = cfs->readdir(dir))) {
 					if(!strncmp(d->name, ".__", 3))
 						continue;
-					link_putfstring(l, "%s\n", stalltime, d->name);
+					link_printf(l, stalltime, "%s\n", d->name);
 				}
 				cfs->closedir(dir);
 				link_putliteral(l, "\n", stalltime);
@@ -789,7 +791,7 @@ static void chirp_handler(struct link *l, const char *addr, const char *subject)
 			time_t transmission_stalltime = time(NULL)+(length/1024)+30; /* 1KB/s minimum */
 			transmission_stalltime = MAX(stalltime, transmission_stalltime);
 
-			link_putfstring(l, "%" PRId64 "\n", transmission_stalltime, length);
+			link_printf(l, transmission_stalltime, "%" PRId64 "\n", length);
 
 			INT64_T total = 0;
 			while (total < length) {
@@ -954,13 +956,6 @@ static void chirp_handler(struct link *l, const char *addr, const char *subject)
 		} else if(sscanf(line, "thirdput %s %s %s", path, chararg1, newpath) == 3) {
 			const char *hostname = chararg1;
 			path_fix(path);
-			if (cfs == &chirp_fs_confuga) {
-				/* Confuga cannot support thirdput because of auth problems,
-				 * see Authentication comment in chirp_receive.
-				 */
-				errno = EACCES;
-				goto failure;
-			}
 			/* ACL check will occur inside of chirp_thirdput */
 			result = chirp_thirdput(subject, path, hostname, newpath, stalltime);
 		} else if(sscanf(line, "open %s %s %" SCNd64, path, newpath, &mode) == 3) {
@@ -1476,10 +1471,10 @@ static void chirp_handler(struct link *l, const char *addr, const char *subject)
 				char *key;
 				struct chirp_audit *entry;
 
-				link_putfstring(l, "%d\n", stalltime, hash_table_size(table));
+				link_printf(l, stalltime, "%d\n", hash_table_size(table));
 				hash_table_firstkey(table);
 				while(hash_table_nextkey(table, &key, (void *) &entry)) {
-					link_putfstring(l, "%s %" PRId64 " %" PRId64 " %" PRId64 "\n", stalltime, key, entry->nfiles, entry->ndirs, entry->nbytes);
+					link_printf(l, stalltime, "%s %" PRId64 " %" PRId64 " %" PRId64 "\n", key, entry->nfiles, entry->ndirs, entry->nbytes);
 				}
 				chirp_audit_delete(table);
 				result = 0;
@@ -1544,9 +1539,9 @@ static void chirp_handler(struct link *l, const char *addr, const char *subject)
 				path_fix(fixed);
 
 				if(cfs->access(fixed, F_OK) == -1) {
-					link_putfstring(l, "%d:%d:%s:\n", stalltime, ENOENT, CHIRP_SEARCH_ERR_OPEN, fixed);
+					link_printf(l, stalltime, "%d:%d:%s:\n", ENOENT, CHIRP_SEARCH_ERR_OPEN, fixed);
 				} else if(!chirp_acl_check(fixed, subject, CHIRP_ACL_WRITE)) {
-					link_putfstring(l, "%d:%d:%s:\n", stalltime, EPERM, CHIRP_SEARCH_ERR_OPEN, fixed);
+					link_printf(l, stalltime, "%d:%d:%s:\n", EPERM, CHIRP_SEARCH_ERR_OPEN, fixed);
 				} else {
 					int found = cfs->search(subject, fixed, pattern, flags, l, stalltime);
 					if(found && (flags & CHIRP_SEARCH_STOPATFIRST))
@@ -1566,10 +1561,10 @@ static void chirp_handler(struct link *l, const char *addr, const char *subject)
 			if ((length = getvarstring(l, stalltime, buffer, length, 0)) == -1)
 				goto failure;
 			debug(D_CHIRP, "--> job_create `%.*s'", (int)length, (char *)buffer);
-			json_value *J = json_parse(buffer, length);
-			if (J) {
-				result = chirp_job_create(&id, J, esubject);
-				json_value_free(J);
+			struct jx *j = jx_parse_string_and_length(buffer, length);
+			if (j) {
+				result = chirp_job_create(&id, j, esubject);
+				jx_delete(j);
 				if (result) {
 					errno = result;
 					goto failure;
@@ -1580,58 +1575,20 @@ static void chirp_handler(struct link *l, const char *addr, const char *subject)
 				errno = EINVAL;
 				goto failure;
 			}
-		} else if(sscanf(line, "job_commit %" PRId64, &length) == 1) {
-			if ((length = getvarstring(l, stalltime, buffer, length, 0)) == -1)
+		} else if(sscanf(line, "job_commit %" PRICHIRP_JOBID_T, &id) == 1) {
+			debug(D_CHIRP, "--> job_commit %" PRICHIRP_JOBID_T,id);
+			result = chirp_job_commit(id, esubject);
+		} else if(sscanf(line, "job_kill %" PRICHIRP_JOBID_T, &id) == 1) {
+			debug(D_CHIRP, "--> job_kill %" PRICHIRP_JOBID_T,id);
+			result = chirp_job_kill(id, esubject);
+		} else if(sscanf(line, "job_status %" PRICHIRP_JOBID_T, &id) == 1) {
+			debug(D_CHIRP, "--> job_status %" PRICHIRP_JOBID_T, id);
+			result = chirp_job_status(id, esubject, B);
+			if (result) {
+				errno = result;
 				goto failure;
-			debug(D_CHIRP, "--> job_commit `%.*s'", (int)length, (char *)buffer);
-			json_value *J = json_parse(buffer, length);
-			if (J) {
-				result = chirp_job_commit(J, esubject);
-				json_value_free(J);
-				if (result) {
-					errno = result;
-					goto failure;
-				}
 			} else {
-				debug(D_DEBUG, "does not parse as json!");
-				errno = EINVAL;
-				goto failure;
-			}
-		} else if(sscanf(line, "job_kill %" PRId64, &length) == 1) {
-			if ((length = getvarstring(l, stalltime, buffer, length, 0)) == -1)
-				goto failure;
-			debug(D_DEBUG, "--> job_kill `%.*s'", (int)length, (char *)buffer);
-			json_value *J = json_parse(buffer, length);
-			if (J) {
-				result = chirp_job_kill(J, esubject);
-				json_value_free(J);
-				if (result) {
-					errno = result;
-					goto failure;
-				}
-			} else {
-				debug(D_DEBUG, "does not parse as json!");
-				errno = EINVAL;
-				goto failure;
-			}
-		} else if(sscanf(line, "job_status %" PRId64, &length) == 1) {
-			if ((length = getvarstring(l, stalltime, buffer, length, 0)) == -1)
-				goto failure;
-			debug(D_CHIRP, "--> job_status `%.*s'", (int)length, (char *)buffer);
-			json_value *J = json_parse(buffer, length);
-			if (J) {
-				result = chirp_job_status(J, esubject, B);
-				if (result) {
-					errno = result;
-					goto failure;
-				} else {
-					result = buffer_pos(B);
-				}
-				json_value_free(J);
-			} else {
-				debug(D_DEBUG, "does not parse as json!");
-				errno = EINVAL;
-				goto failure;
+				result = buffer_pos(B);
 			}
 		} else if(sscanf(line, "job_wait %" SCNCHIRP_JOBID_T " %" SCNd64, &id, &length) == 2) {
 			result = chirp_job_wait(id, esubject, length, B);
@@ -1641,23 +1598,9 @@ static void chirp_handler(struct link *l, const char *addr, const char *subject)
 			} else {
 				result = buffer_pos(B);
 			}
-		} else if(sscanf(line, "job_reap %" PRId64, &length) == 1) {
-			if ((length = getvarstring(l, stalltime, buffer, length, 0)) == -1)
-				goto failure;
-			debug(D_DEBUG, "--> job_reap `%.*s'", (int)length, (char *)buffer);
-			json_value *J = json_parse(buffer, length);
-			if (J) {
-				result = chirp_job_reap(J, esubject);
-				json_value_free(J);
-				if (result) {
-					errno = result;
-					goto failure;
-				}
-			} else {
-				debug(D_DEBUG, "does not parse as json!");
-				errno = EINVAL;
-				goto failure;
-			}
+		} else if(sscanf(line, "job_reap %" PRICHIRP_JOBID_T, &id) == 1) {
+			debug(D_DEBUG, "--> job_reap %" PRICHIRP_JOBID_T, id);
+			result = chirp_job_reap(id, esubject);
 		} else {
 			errno = ENOSYS;
 			goto failure;
@@ -1669,7 +1612,7 @@ failure:
 result:
 		if (result < 0)
 			result = errno_to_chirp(errno);
-		if (link_putfstring(l, "%" PRId64 "\n", stalltime, result) == -1)
+		if (link_printf(l, stalltime, "%" PRId64 "\n", result) == -1)
 			goto die;
 		if(result >= 0 && buffer_pos(B)) {
 			if (link_putlstring(l, buffer_tostring(B), buffer_pos(B), stalltime) == -1)
@@ -1752,17 +1695,14 @@ static void chirp_receive(struct link *link, char url[CHIRP_PATH_MAX])
 
 		downgrade(); /* downgrade privileges after authentication */
 
-		/* See above comment concerning authentication. */
-		if (cfs != &chirp_fs_confuga) {
-			/* Enable only globus, hostname, and address authentication for third-party transfers. */
-			auth_clear();
-			if(auth_globus_has_delegated_credential()) {
-				auth_globus_use_delegated_credential(1);
-				auth_globus_register();
-			}
-			auth_hostname_register();
-			auth_address_register();
+		/* Enable only globus, hostname, and address authentication for third-party transfers. */
+		auth_clear();
+		if(auth_globus_has_delegated_credential()) {
+			auth_globus_use_delegated_credential(1);
+			auth_globus_register();
 		}
+		auth_hostname_register();
+		auth_address_register();
 
 		change_process_title("chirp_server [%s:%d] [%s]", addr, port, typesubject);
 
@@ -1846,7 +1786,7 @@ static void show_help(const char *cmd)
 	fprintf(stdout, "The most common options are:\n");
 	fprintf(stdout, " %-30s URL of storage directory, like `file://path' or `hdfs://host:port/path'.\n", "-r,--root=<url>");
 	fprintf(stdout, " %-30s Enable debugging for this subsystem.\n", "-d,--debug=<name>");
-	fprintf(stdout, " %-30s Send debugging to this file. (can also be :stderr, :stdout, :syslog, or :journal)\n", "-o,--debug-file=<file>");
+	fprintf(stdout, " %-30s Send debugging to this file. (can also be :stderr, or :stdout)\n", "-o,--debug-file=<file>");
 	fprintf(stdout, " %-30s Send status updates to this host. (default: `%s')\n", "-u,--advertise=<host>", CATALOG_HOST);
 	fprintf(stdout, " %-30s Show version info.\n", "-v,--version");
 	fprintf(stdout, " %-30s This message.\n", "-h,--help");
@@ -2283,7 +2223,7 @@ int main(int argc, char *argv[])
 			gc_alarm = time(0) + GC_TIMEOUT;
 		}
 
-		/* Wait for action on one of two ports: the master TCP port, or the internal pipe. */
+		/* Wait for action on one of two ports: the main TCP port, or the internal pipe. */
 		/* If the limit of child procs has been reached, don't watch the TCP port. */
 
 		fd_set rfds;
@@ -2334,4 +2274,4 @@ int main(int argc, char *argv[])
 	}
 }
 
-/* vim: set noexpandtab tabstop=4: */
+/* vim: set noexpandtab tabstop=8: */
