@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2003-2004 Douglas Thain and the University of Wisconsin
-Copyright (C) 2005- The University of Notre Dame
+Copyright (C) 2022 The University of Notre Dame
 This software is distributed under the GNU General Public License.
 See the file COPYING for details.
 */
@@ -16,7 +16,7 @@ doing the logging, so that various subsystems may be easily turned on and off.
 For example, the Chirp subsystem has many statements like this:
 
 <pre>
-debug(D_CHIRP,"reading file %s from host %s:d",filename,hostname,port);
+debug(D_CHIRP,"reading file %s from host %s:%d",filename,hostname,port);
 </pre>
 
 The <tt>main</tt> routine of a program is responsible for
@@ -93,6 +93,10 @@ unless it has the flags D_NOTICE or D_FATAL.  For example, a main program might
 #define D_JX       (1LL<<45)  /**< Debug JX */
 #define D_MAKEFLOW_HOOK  (1LL<<46)  /**< Debug makeflow's hook system */
 #define D_EXT      (1LL<<47)  /**< Debug the ext module in Parrot */
+#define D_VINE  (1LL<<48) /**< Debug the taskvine service. */
+#define D_TLQ (1LL<<49) /**< Debug the TLQ service's interactions with CCTools. */
+#define D_SSL (1LL<<50) /**< Debug the TLQ service's interactions with CCTools. */
+#define D_BUCKETING (1LL<<51) /**< Debug bucketing algorithms. */
 
 /** Debug all remote I/O operations. */
 #define D_REMOTE   (D_HTTP|D_FTP|D_NEST|D_CHIRP|D_DCAP|D_RFIO|D_LFC|D_GFAL|D_MULTI|D_GROW|D_IRODS|D_HDFS|D_BXGRID|D_XROOTD|D_CVMFS)
@@ -110,6 +114,8 @@ than change all of our code, we simply insert these defines to transparently
 modify the linker namespace we are using.
 */
 
+#ifndef DOXYGEN
+
 #define debug                  cctools_debug
 #define fatal                  cctools_fatal
 #define notice                 cctools_notice
@@ -126,6 +132,8 @@ modify the linker namespace we are using.
 #define debug_set_flag_name    cctools_debug_set_flag_name
 #define debug_rename           cctools_debug_rename
 
+#endif
+
 /** Emit a debugging message.
 Logs a debugging message, if the given flags are active.
 @param flags Any of the standard debugging flags OR-ed together.
@@ -247,6 +255,10 @@ void debug_rename(const char *suffix);
 */
 void debug_reopen(void);
 
+/** Close the debug stream (only for disk files, no stderr or stdout).
+*/
+void debug_close(void);
+
 /* LDEBUG likes debug, but also print the code location.
  * NOTE: the caller of this macro must supply at least one argument after the format string.
  */
