@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2013- The University of Notre Dame
+  Copyright (C) 2022 The University of Notre Dame
   This software is distributed under the GNU General Public License.
   See the file COPYING for details.
 */
@@ -36,33 +36,50 @@
 #include <sched.h>
 #endif
 
-#include "timestamp.h"
 #include "itable.h"
+#include "timestamp.h"
 
 #define CCTOOLS_HELPER_DEBUG_MESSAGES 0
 
 #define D_RMON stderr
-#define debug if(CCTOOLS_HELPER_DEBUG_MESSAGES) fprintf
+#define debug \
+	if (CCTOOLS_HELPER_DEBUG_MESSAGES) \
+	fprintf
 
 #include "rmonitor_helper_comm.h"
 
 #define BUFFER_MAX 1024
 
-#define PUSH_ERRNO { int last_errno = errno; errno = 0;
-#define POP_ERRNO(msg) msg.error = errno; if(!errno){ errno = last_errno; } }
+#define PUSH_ERRNO \
+	{ \
+		int last_errno = errno; \
+		errno = 0;
+#define POP_ERRNO(msg) \
+	msg.error = errno; \
+	if (!errno) { \
+		errno = last_errno; \
+	} \
+	}
 
-#define START(msg) { if(msg.type == RX || msg.type == TX) msg.start = timestamp_get(); PUSH_ERRNO
-#define END(msg)   POP_ERRNO(msg) if(msg.type == RX || msg.type == TX) msg.end = timestamp_get(); }
+#define START(msg) \
+	{ \
+		if (msg.type == RX || msg.type == TX) \
+			msg.start = timestamp_get(); \
+		PUSH_ERRNO
+#define END(msg) \
+	POP_ERRNO(msg) \
+	if (msg.type == RX || msg.type == TX) \
+		msg.end = timestamp_get(); \
+	}
 
 static struct itable *family_of_fd = NULL;
 static uint64_t start_time = 0;
-static uint64_t end_time   = 0;
+static uint64_t end_time = 0;
 
 static int stop_short_running = 0; /* Stop processes that run for less than RESOURCE_MONITOR_SHORT_TIME seconds. */
-static int root_process       = 0;  /* 1 for first process to be monitored. */
 
-#define declare_original_dlsym(name) __typeof__(name) *original_ ## name;
-#define define_original_dlsym(name) original_ ## name = dlsym(RTLD_NEXT, #name);
+#define declare_original_dlsym(name) __typeof__(name) *original_##name;
+#define define_original_dlsym(name) original_##name = dlsym(RTLD_NEXT, #name);
 
 declare_original_dlsym(fork);
 declare_original_dlsym(chdir);
@@ -87,9 +104,10 @@ declare_original_dlsym(open64);
 
 static int initializing_helper = 0;
 
-void rmonitor_helper_initialize() {
+void rmonitor_helper_initialize()
+{
 
-	if(initializing_helper)
+	if (initializing_helper)
 		return;
 
 	initializing_helper = 1;
@@ -115,18 +133,11 @@ void rmonitor_helper_initialize() {
 	define_original_dlsym(open64);
 #endif
 
-	if(!family_of_fd) {
+	if (!family_of_fd) {
 		family_of_fd = itable_create(8);
 	}
 
-	if(getenv(RESOURCE_MONITOR_ROOT_PROCESS)) {
-		root_process = 1;
-		unsetenv(RESOURCE_MONITOR_ROOT_PROCESS);
-	} else {
-		root_process = 0;
-	}
-
-	if(getenv(RESOURCE_MONITOR_HELPER_STOP_SHORT)) {
+	if (getenv(RESOURCE_MONITOR_HELPER_STOP_SHORT)) {
 		stop_short_running = 1;
 	} else {
 		stop_short_running = 0;
@@ -135,12 +146,17 @@ void rmonitor_helper_initialize() {
 	initializing_helper = 0;
 }
 
+int is_root_process()
+{
+	const char *pid_s = getenv(RESOURCE_MONITOR_ROOT_PROCESS);
+	return (pid_s && atoi(pid_s) == getpid());
+}
 
 pid_t fork()
 {
 	pid_t pid;
 
-	if(!original_fork) {
+	if (!original_fork) {
 		rmonitor_helper_initialize();
 		assert(original_fork);
 	}
@@ -148,17 +164,16 @@ pid_t fork()
 	debug(D_RMON, "fork from %d.\n", getpid());
 	pid = original_fork();
 
-	if(!pid)
-	{
+	if (!pid) {
 		char start_tmp[256];
 		snprintf(start_tmp, 256, "%" PRId64, timestamp_get());
 		setenv(RESOURCE_MONITOR_PROCESS_START, start_tmp, 1);
 
 		struct rmonitor_msg msg;
-		msg.type   = BRANCH;
+		msg.type = BRANCH;
 
 		/* We only send a message from the child, thus error is always zero. */
-		msg.error  = 0;
+		msg.error = 0;
 		msg.origin = getpid();
 		msg.data.p = getppid();
 
@@ -183,27 +198,25 @@ pid_t __vfork()
 	return fork();
 }
 
-
 int chdir(const char *path)
 {
-	int   status;
+	int status;
 
-	if(!original_chdir) {
+	if (!original_chdir) {
 		return syscall(SYS_chdir, path);
 	}
 
 	debug(D_RMON, "chdir from %d.\n", getpid());
 	status = original_chdir(path);
 
-	if(status == 0)
-	{
+	if (status == 0) {
 		struct rmonitor_msg msg;
-		char  *newpath = getcwd(NULL, 0);
+		char *newpath = getcwd(NULL, 0);
 
-		msg.type   = CHDIR;
+		msg.type = CHDIR;
 
 		/* We only send a message when cwd actually changes, so errno is always 0. */
-		msg.error  = 0;
+		msg.error = 0;
 		msg.origin = getpid();
 		strcpy(msg.data.s, newpath);
 		free(newpath);
@@ -216,23 +229,22 @@ int chdir(const char *path)
 
 int fchdir(int fd)
 {
-	int   status;
-	if(!original_fchdir) {
+	int status;
+	if (!original_fchdir) {
 		return syscall(SYS_fchdir, fd);
 	}
 
 	debug(D_RMON, "fchdir from %d.\n", getpid());
 	status = original_fchdir(fd);
 
-	if(status == 0)
-	{
+	if (status == 0) {
 		struct rmonitor_msg msg;
-		char  *newpath = getcwd(NULL, 0);
+		char *newpath = getcwd(NULL, 0);
 
-		msg.type   = CHDIR;
+		msg.type = CHDIR;
 
 		/* We only send a message when cwd actually changes, so errno is always 0. */
-		msg.error  = 0;
+		msg.error = 0;
 		msg.origin = getpid();
 		strcpy(msg.data.s, newpath);
 		free(newpath);
@@ -245,11 +257,11 @@ int fchdir(int fd)
 
 int close(int fd)
 {
-	if(!original_close) {
+	if (!original_close) {
 		return syscall(SYS_close, fd);
 	}
 
-	if(family_of_fd) {
+	if (family_of_fd) {
 		itable_remove(family_of_fd, fd);
 	}
 
@@ -258,12 +270,13 @@ int close(int fd)
 	return status;
 }
 
-static int open_for_writing(int fd) {
+static int open_for_writing(int fd)
+{
 	int flags, access_mode;
 
-	flags       = fcntl(fd, F_GETFL);
+	flags = fcntl(fd, F_GETFL);
 
-	if(flags == -1) {
+	if (flags == -1) {
 		/* if error, assume output. */
 		return 1;
 	}
@@ -277,32 +290,32 @@ int open(const char *path, int flags, ...)
 	struct rmonitor_msg msg;
 
 	va_list ap;
-	int     fd;
-	int     mode;
+	int fd;
+	int mode;
 
 	va_start(ap, flags);
 	mode = va_arg(ap, int);
 	va_end(ap);
 
-	if(!original_open) {
+	if (!original_open) {
 		return syscall(SYS_open, path, flags, mode);
 	}
 
 	debug(D_RMON, "open %s from %d.\n", path, getpid());
 
 	START(msg)
-		fd = original_open(path, flags, mode);
+	fd = original_open(path, flags, mode);
 	END(msg)
 
 	/* With ENOENT we do not send a message, simply to reduce spam. */
-	if(msg.error == ENOENT)
+	if (msg.error == ENOENT)
 		return fd;
 
 	/* Consider file as input by default. */
-	msg.type   = OPEN_INPUT;
+	msg.type = OPEN_INPUT;
 
-	if(fd > -1 && open_for_writing(fd)) {
-		msg.type   = OPEN_OUTPUT;
+	if (fd > -1 && open_for_writing(fd)) {
+		msg.type = OPEN_OUTPUT;
 	}
 
 	msg.origin = getpid();
@@ -320,32 +333,32 @@ int open64(const char *path, int flags, ...)
 	struct rmonitor_msg msg;
 
 	va_list ap;
-	int     fd;
-	int     mode;
+	int fd;
+	int mode;
 
 	va_start(ap, flags);
 	mode = va_arg(ap, int);
 	va_end(ap);
 
-	if(!original_open64) {
+	if (!original_open64) {
 		return syscall(SYS_open, path, flags | O_LARGEFILE, mode);
 	}
 
 	debug(D_RMON, "open64 %s from %d.\n", path, getpid());
 
 	START(msg)
-		fd = original_open64(path, flags, mode);
+	fd = original_open64(path, flags, mode);
 	END(msg)
 
 	/* With ENOENT we do not send a message, simply to reduce spam. */
-	if(msg.error == ENOENT)
+	if (msg.error == ENOENT)
 		return fd;
 
 	/* Consider file as input by default. */
-	msg.type   = OPEN_INPUT;
+	msg.type = OPEN_INPUT;
 
-	if(fd > -1 && open_for_writing(fd)) {
-		msg.type   = OPEN_OUTPUT;
+	if (fd > -1 && open_for_writing(fd)) {
+		msg.type = OPEN_OUTPUT;
 	}
 
 	msg.origin = getpid();
@@ -361,15 +374,15 @@ int socket(int domain, int type, int protocol)
 {
 	int fd;
 
-	if(!original_socket) {
+	if (!original_socket) {
 		rmonitor_helper_initialize();
 		assert(original_socket);
 	}
 
 	fd = original_socket(domain, type, protocol);
 
-	if(fd > -1 && (domain != AF_LOCAL || domain != AF_NETLINK)) {
-		itable_insert(family_of_fd, fd, (void **) 1);
+	if (fd > -1 && (domain != AF_LOCAL || domain != AF_NETLINK)) {
+		itable_insert(family_of_fd, fd, (void **)1);
 	} else {
 		itable_remove(family_of_fd, fd);
 	}
@@ -381,21 +394,21 @@ ssize_t write(int fd, const void *buf, size_t count)
 {
 	struct rmonitor_msg msg;
 
-	if(!original_write) {
+	if (!original_write) {
 		return syscall(SYS_write, fd, buf, count);
 	}
 
 	msg.origin = getpid();
 
-	if(family_of_fd && itable_lookup(family_of_fd, fd)) {
-		msg.type   = TX;
+	if (family_of_fd && itable_lookup(family_of_fd, fd)) {
+		msg.type = TX;
 	} else {
-		msg.type   = WRITE;
+		msg.type = WRITE;
 	}
 
 	ssize_t real_count;
 	START(msg)
-		real_count = original_write(fd, buf, count);
+	real_count = original_write(fd, buf, count);
 	END(msg)
 
 	msg.data.n = real_count;
@@ -408,21 +421,21 @@ ssize_t read(int fd, void *buf, size_t count)
 {
 	struct rmonitor_msg msg;
 
-	if(!original_read) {
+	if (!original_read) {
 		return syscall(SYS_read, fd, buf, count);
 	}
 
 	msg.origin = getpid();
 
-	if(family_of_fd && itable_lookup(family_of_fd, fd)) {
-		msg.type   = RX;
+	if (family_of_fd && itable_lookup(family_of_fd, fd)) {
+		msg.type = RX;
 	} else {
-		msg.type   = READ;
+		msg.type = READ;
 	}
 
 	ssize_t real_count;
 	START(msg)
-		real_count = original_read(fd, buf, count);
+	real_count = original_read(fd, buf, count);
 	END(msg)
 
 	msg.data.n = real_count;
@@ -435,17 +448,17 @@ ssize_t recv(int fd, void *buf, size_t count, int flags)
 {
 	struct rmonitor_msg msg;
 
-	if(!original_recv) {
+	if (!original_recv) {
 		rmonitor_helper_initialize();
 		assert(original_recv);
 	}
 
-	msg.type   = RX;
+	msg.type = RX;
 	msg.origin = getpid();
 
 	ssize_t real_count;
 	START(msg)
-		real_count = original_recv(fd, buf, count, flags);
+	real_count = original_recv(fd, buf, count, flags);
 	END(msg)
 
 	msg.data.n = real_count;
@@ -458,17 +471,17 @@ ssize_t recvfrom(int fd, void *buf, size_t count, int flags, struct sockaddr *sr
 {
 	struct rmonitor_msg msg;
 
-	if(!original_recvfrom) {
+	if (!original_recvfrom) {
 		rmonitor_helper_initialize();
 		assert(original_recvfrom);
 	}
 
-	msg.type   = RX;
+	msg.type = RX;
 	msg.origin = getpid();
 
 	ssize_t real_count;
 	START(msg)
-		real_count = original_recvfrom(fd, buf, count, flags, src, addrlen);
+	real_count = original_recvfrom(fd, buf, count, flags, src, addrlen);
 	END(msg)
 
 	msg.data.n = real_count;
@@ -481,17 +494,17 @@ ssize_t send(int fd, const void *buf, size_t count, int flags)
 {
 	struct rmonitor_msg msg;
 
-	if(!original_send) {
+	if (!original_send) {
 		rmonitor_helper_initialize();
 		assert(original_send);
 	}
 
-	msg.type   = TX;
+	msg.type = TX;
 	msg.origin = getpid();
 
 	ssize_t real_count;
 	START(msg)
-		real_count = original_send(fd, buf, count, flags);
+	real_count = original_send(fd, buf, count, flags);
 	END(msg)
 
 	msg.data.n = real_count;
@@ -504,17 +517,17 @@ ssize_t sendmsg(int fd, const struct msghdr *mg, int flags)
 {
 	struct rmonitor_msg msg;
 
-	if(!original_sendmsg) {
+	if (!original_sendmsg) {
 		rmonitor_helper_initialize();
 		assert(original_sendmsg);
 	}
 
-	msg.type   = TX;
+	msg.type = TX;
 	msg.origin = getpid();
 
 	ssize_t real_count;
 	START(msg)
-		real_count = original_sendmsg(fd, mg, flags);
+	real_count = original_sendmsg(fd, mg, flags);
 	END(msg)
 
 	msg.data.n = real_count;
@@ -523,22 +536,21 @@ ssize_t sendmsg(int fd, const struct msghdr *mg, int flags)
 	return real_count;
 }
 
-
 ssize_t recvmsg(int fd, struct msghdr *mg, int flags)
 {
 	struct rmonitor_msg msg;
 
-	if(!original_recvmsg) {
+	if (!original_recvmsg) {
 		rmonitor_helper_initialize();
 		assert(original_recvmsg);
 	}
 
-	msg.type   = RX;
+	msg.type = RX;
 	msg.origin = getpid();
 
 	ssize_t real_count;
 	START(msg)
-		real_count = original_recvmsg(fd, mg, flags);
+	real_count = original_recvmsg(fd, mg, flags);
 	END(msg)
 
 	msg.data.n = real_count;
@@ -549,7 +561,8 @@ ssize_t recvmsg(int fd, struct msghdr *mg, int flags)
 
 /* dummy handler. In RH5, SIGCONT is ignored by sigprocmask/sigtimedwait,
  * unless handler is different than SIG_IGN. */
-void exit_signal_handler(int signum) {
+void exit_signal_handler(int signum)
+{
 	return;
 }
 
@@ -557,7 +570,7 @@ void exit_wrapper_preamble(int status)
 {
 	static int did_exit_wrapper = 0;
 
-	if(did_exit_wrapper)
+	if (did_exit_wrapper)
 		return;
 
 	did_exit_wrapper = 1;
@@ -575,23 +588,24 @@ void exit_wrapper_preamble(int status)
 	end_time = timestamp_get();
 
 	struct rmonitor_msg msg;
-	msg.type   = END_WAIT;
-	msg.error  = 0;
+	msg.type = END_WAIT;
+	msg.error = 0;
 	msg.origin = getpid();
 	msg.data.n = status;
-	msg.start  = start_time;
-	msg.end    = end_time;;
+	msg.start = start_time;
+	msg.end = end_time;
+	;
 
 	sighandler_t old_handler = signal(SIGCONT, exit_signal_handler);
 
 	int short_process = 0;
-	if(root_process) {
+	if (is_root_process()) {
 		// root process is never considered a short running process
 		short_process = 0;
-	} else if(stop_short_running) {
+	} else if (stop_short_running) {
 		// we are stopping all processes, so no process is considered short running
 		short_process = 0;
-	} else if(end_time < (start_time + RESOURCE_MONITOR_SHORT_TIME)) {
+	} else if (end_time < (start_time + RESOURCE_MONITOR_SHORT_TIME)) {
 		// process ran for less than RESOURCE_MONITOR_SHORT_TIME, so it is
 		// considered short running.
 		short_process = 1;
@@ -602,15 +616,15 @@ void exit_wrapper_preamble(int status)
 
 	// If not short running, stop the process for examination.
 	int blocking_signals = 0;
-	if(!short_process) {
-		if(sigprocmask(SIG_SETMASK, &all_signals, &old_signals) != -1) {
+	if (!short_process) {
+		if (sigprocmask(SIG_SETMASK, &all_signals, &old_signals) != -1) {
 			blocking_signals = 1;
 		}
 	}
 
 	send_monitor_msg(&msg);
 
-	if(blocking_signals) {
+	if (blocking_signals) {
 		debug(D_RMON, "Waiting for monitoring: %d.\n", getpid());
 		sigtimedwait(&all_signals, NULL, &timeout);
 		sigprocmask(SIG_SETMASK, &old_signals, NULL);
@@ -627,20 +641,20 @@ void end_wrapper_epilogue(void)
 	debug(D_RMON, "%s from %d.\n", str_msgtype(END), getpid());
 
 	struct rmonitor_msg msg;
-	msg.type   = END;
-	msg.error  = 0;
+	msg.type = END;
+	msg.error = 0;
 	msg.origin = getpid();
 	msg.data.p = getpid();
-	msg.start  = start_time;
-	msg.end    = end_time;;
+	msg.start = start_time;
+	msg.end = end_time;
+	;
 
 	send_monitor_msg(&msg);
 }
 
-
 void exit(int status)
 {
-	if(!original_exit){
+	if (!original_exit) {
 		syscall(SYS_exit, status);
 	}
 
@@ -664,7 +678,7 @@ void _exit(int status)
 	   will be ignored as the processes would no longer in the
 	   monitoring tables. */
 
-	if(!original_exit){
+	if (!original_exit) {
 		syscall(SYS_exit, status);
 	}
 
@@ -683,10 +697,10 @@ void _exit(int status)
 
 pid_t waitpid(pid_t pid, int *status, int options)
 {
-	int status_; //status might be NULL, thus we use status_ to retrive the state.
+	int status_; // status might be NULL, thus we use status_ to retrive the state.
 	pid_t pidb;
 
-	if(!original_waitpid) {
+	if (!original_waitpid) {
 		rmonitor_helper_initialize();
 		assert(original_waitpid);
 	}
@@ -694,18 +708,17 @@ pid_t waitpid(pid_t pid, int *status, int options)
 	debug(D_RMON, "waiting from %d for %d.\n", getpid(), pid);
 	pidb = original_waitpid(pid, &status_, options);
 
-	if(WIFEXITED(status_) || WIFSIGNALED(status_))
-	{
+	if (WIFEXITED(status_) || WIFSIGNALED(status_)) {
 		struct rmonitor_msg msg;
-		msg.type   = WAIT;
-		msg.error  = 0;          /* send message only on success, so error is 0. */
+		msg.type = WAIT;
+		msg.error = 0; /* send message only on success, so error is 0. */
 		msg.origin = getpid();
 		msg.data.p = pidb;
 
 		send_monitor_msg(&msg);
 	}
 
-	if(status)
+	if (status)
 		*status = status_;
 
 	return pidb;
@@ -718,14 +731,16 @@ pid_t wait(int *status)
 
 #if defined(__clang__) || defined(__GNUC__)
 
-void __attribute__((constructor)) init() {
+void __attribute__((constructor)) init()
+{
 	/* find the dlsym values when loading the library. */
 	rmonitor_helper_initialize();
 }
 
 /* wrap main ensures exit_wrapper_preamble runs, and thus monitoring
 is done at least once */
-void __attribute__((destructor)) fini() {
+void __attribute__((destructor)) fini()
+{
 	/* we use default status of 0, since if command did not call exit
 	 * explicitely, that is the default. */
 	exit_wrapper_preamble(0);
