@@ -3,17 +3,23 @@
 #include "work_queue.h"
 #include "work_queue_internal.h"
 #include "work_queue_gpus.h"
+#include "work_queue_protocol.h"
+#include "work_queue_coprocess.h"
 
 #include "debug.h"
 #include "errno.h"
 #include "macros.h"
 #include "stringtools.h"
 #include "create_dir.h"
-#include "delete_dir.h"
 #include "list.h"
 #include "disk_alloc.h"
 #include "path.h"
 #include "xxmalloc.h"
+#include "trash.h"
+#include "link.h"
+#include "timestamp.h"
+#include "domain_name.h"
+#include "full_io.h"
 
 #include <math.h>
 #include <stdio.h>
@@ -30,26 +36,39 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#define SMALL_BUFFER_SIZE 256
-#define MAX_BUFFER_SIZE 4096
-#define DOCKER_WORK_DIR "/home/worker"
-#define TMP_SCRIPT "tmp.sh"
-#define DEFAULT_EXE_APP "#!/bin/sh"
+/*
+Create the task sandbox directory.  If disk allocation is enabled,
+make an allocation, otherwise just make a directory.
+Create temporary directories inside as well.
+*/
 
-// return 0 on error, 1 otherwise
-static int create_task_directories(struct work_queue_process *p) {
-	char tmpdir_template[1024];
+extern char * workspace;
 
-	p->sandbox = string_format("t.%d", p->task->taskid);
-	if(!create_dir(p->sandbox, 0777)) {
-		return 0;
+static int create_sandbox_dir( struct work_queue_process *p, int disk_allocation )
+{
+	p->cache_dir = string_format("%s/cache",workspace);
+  	p->sandbox = string_format("%s/t.%d", workspace,p->task->taskid);
+
+	if(disk_allocation) {
+		work_queue_process_compute_disk_needed(p);
+		if(p->task->resources_requested->disk > 0) {
+			int64_t size = (p->task->resources_requested->disk) * 1024;
+			if(disk_alloc_create(p->sandbox, "ext2", size) == 0) {
+				p->loop_mount = 1;
+				debug(D_WQ, "allocated %"PRId64"MB in %s\n",size,p->sandbox);
+				// keep going and fall through
+			} else {
+				debug(D_WQ, "couldn't allocate %"PRId64"MB in %s\n",size,p->sandbox);
+				return 0;
+			}
+		} else {
+			if(!create_dir(p->sandbox, 0777)) return 0;
+		}
+	} else {
+		if(!create_dir(p->sandbox, 0777)) return 0;
 	}
 
-	char absolute[1024];
-	path_absolute(p->sandbox, absolute, 1);
-	free(p->sandbox);
-	p->sandbox = xxstrdup(absolute);
-
+	char tmpdir_template[1024];
 	string_nformat(tmpdir_template, sizeof(tmpdir_template), "%s/cctools-temp-t.%d.XXXXXX", p->sandbox, p->task->taskid);
 	if(mkdtemp(tmpdir_template) == NULL) {
 		return 0;
@@ -63,49 +82,30 @@ static int create_task_directories(struct work_queue_process *p) {
 	return 1;
 }
 
+/*
+Create a work_queue_process and all of the information necessary for invocation.
+However, do not allocate substantial resources at this point.
+*/
+
 struct work_queue_process *work_queue_process_create(struct work_queue_task *wq_task, int disk_allocation)
 {
 	struct work_queue_process *p = malloc(sizeof(*p));
 	memset(p, 0, sizeof(*p));
 	p->task = wq_task;
 	p->task->disk_allocation_exhausted = 0;
-	//placeholder filesystem until permanent solution
-	char *fs = "ext2";
+	p->coprocess = NULL;
 
-	if(disk_allocation == 1) {
-		work_queue_process_compute_disk_needed(p);
-		if(p->task->resources_requested->disk > 0) {
-			int64_t size = (p->task->resources_requested->disk) * 1024;
-			p->sandbox = string_format("t.%d", p->task->taskid);
 
-			if(disk_alloc_create(p->sandbox, fs, size) == 0) {
-				p->loop_mount = 1;
-				debug(D_WQ, "disk_alloc: %"PRId64"MB\n", size);
-				return p;
-			}
-		}
-		if(!create_task_directories(p)) {
-			work_queue_process_delete(p);
-			return 0;
-		}
-
-		p->loop_mount = 0;
-		return p;
-	}
-	else {
-		if(!create_task_directories(p)) {
-			work_queue_process_delete(p);
-			return 0;
-		}
-
-		p->loop_mount = 0;
-		return p;
+	if(!create_sandbox_dir(p,disk_allocation)) {
+		work_queue_process_delete(p);
+		return 0;
 	}
+	return p;
 }
 
+
 void work_queue_process_delete(struct work_queue_process *p)
 {
-
 	if(p->task)
 		work_queue_task_delete(p->task);
 
@@ -114,16 +114,15 @@ void work_queue_process_delete(struct work_queue_process *p)
 	}
 
 	if(p->output_file_name) {
-		unlink(p->output_file_name);
+		trash_file(p->output_file_name);
 		free(p->output_file_name);
 	}
 
 	if(p->sandbox) {
 		if(p->loop_mount == 1) {
 			disk_alloc_delete(p->sandbox);
-		}
-		else {
-			delete_dir(p->sandbox);
+		} else {
+			trash_file(p->sandbox);
 		}
 		free(p->sandbox);
 	}
@@ -131,6 +130,9 @@ void work_queue_process_delete(struct work_queue_process *p)
 	if(p->tmpdir)
 		free(p->tmpdir);
 
+	if(p->cache_dir)
+		free(p->cache_dir);
+	
 	free(p);
 }
 
@@ -201,7 +203,27 @@ static void specify_resources_vars(struct work_queue_process *p) {
 
 static const char task_output_template[] = "./worker.stdout.XXXXXX";
 
-pid_t work_queue_process_execute(struct work_queue_process *p, int container_mode, ...)
+static char * load_input_file(struct work_queue_task *t) {
+	FILE *fp = fopen("infile", "r");
+	if(!fp) {
+		fatal("coprocess could not open file 'infile' for reading: %s", strerror(errno));
+	}
+
+	fseek(fp, 0L, SEEK_END);
+	size_t fsize = ftell(fp);
+	fseek(fp, 0L, SEEK_SET);
+	// using calloc solves problem of garbage appended to buffer
+	char *buf = calloc(fsize + 1, sizeof(*buf));
+
+	int bytes_read = full_fread(fp, buf, fsize);
+	if(bytes_read < 0) {
+		fatal("error reading file: %s", strerror(errno));
+	}
+
+	return buf;
+}
+
+pid_t work_queue_process_execute(struct work_queue_process *p )
 {
 	// make warning
 
@@ -255,13 +277,28 @@ pid_t work_queue_process_execute(struct work_queue_process *p, int container_mod
 		if(result == -1)
 			fatal("could not dup /dev/null to stdin: %s", strerror(errno));
 
-		result = dup2(p->output_fd, STDOUT_FILENO);
-		if(result == -1)
-			fatal("could not dup pipe to stdout: %s", strerror(errno));
+		if (p->coprocess == NULL) {
+			result = dup2(p->output_fd, STDOUT_FILENO);
+			if(result == -1)
+				fatal("could not dup pipe to stdout: %s", strerror(errno));
 
-		result = dup2(p->output_fd, STDERR_FILENO);
-		if(result == -1)
-			fatal("could not dup pipe to stderr: %s", strerror(errno));
+			result = dup2(p->output_fd, STDERR_FILENO);
+			if(result == -1)
+				fatal("could not dup pipe to stderr: %s", strerror(errno));
+			}
+		else {
+			// load data from input file
+			char *input = load_input_file(p->task);
+
+			// call invoke_coprocess_function
+		 	char *output = work_queue_coprocess_run(p->task->command_line, input, p->coprocess, p->task->taskid);
+			// write data to output file
+			if(output) {
+				full_write(p->output_fd, output, strlen(output));
+			}
+			
+			exit(0);
+		}
 
 		close(p->output_fd);
 
@@ -272,79 +309,9 @@ pid_t work_queue_process_execute(struct work_queue_process *p, int container_mod
 
 		export_environment(p);
 
-		va_list arg_lst;
-		if(container_mode == NONE) {
-			execl("/bin/sh", "sh", "-c", p->task->command_line, (char *) 0);
-			_exit(127);	// Failed to execute the cmd.
+		execl("/bin/sh", "sh", "-c", p->task->command_line, (char *) 0);
+		_exit(127);	// Failed to execute the cmd.
 
-		} else if(container_mode == UMBRELLA) {
-			fatal("UMBRELLA mode have not been implemented\n");
-
-		} else {
-			// Write task command into a shell script
-			char *tmp_ptr = p->task->command_line;
-			int cmd_line_size = 0;
-			while(*(++tmp_ptr) != '\0')
-				cmd_line_size++;
-
-			FILE *script_fn = fopen(TMP_SCRIPT, "w");
-			fprintf(script_fn, "%s\n%s", DEFAULT_EXE_APP, p->task->command_line);
-			fclose(script_fn);
-			chmod(TMP_SCRIPT, 0755);
-
-			uid_t uid = getuid();
-			char uid_str[MAX_BUFFER_SIZE];
-			string_nformat(uid_str, sizeof(uid_str), "%d", uid);
-
-			// Get path to sandbox
-			char curr_wrk_dir[MAX_BUFFER_SIZE];
-			char *wrk_space;
-
-			if((wrk_space = getenv("WORK_QUEUE_SANDBOX")) != NULL) {
-				string_nformat(curr_wrk_dir, sizeof(curr_wrk_dir), "%s/%s", wrk_space, p->sandbox);
-			} else {
-				perror("getenv() error");
-			}
-
-			if(container_mode == DOCKER) {
-				va_start(arg_lst, container_mode);
-				char img_name[MAX_BUFFER_SIZE];
-				strncpy(img_name, va_arg(arg_lst, const char *), MAX_BUFFER_SIZE - 1);
-				va_end(arg_lst);
-
-				char mnt_flg_val[MAX_BUFFER_SIZE];
-				string_nformat(mnt_flg_val, sizeof(mnt_flg_val), "%s:%s", curr_wrk_dir, DOCKER_WORK_DIR);
-				// cmd for running the shell script
-				char run_cmd[SMALL_BUFFER_SIZE];
-				string_nformat(run_cmd, sizeof(run_cmd), "./%s", TMP_SCRIPT);
-
-				execl("/usr/bin/docker", "/usr/bin/docker", "run", "--rm", "-v", mnt_flg_val, "-w", DOCKER_WORK_DIR, "-u", uid_str, "-m", "1g", img_name, run_cmd, (char *) 0);
-				_exit(127);	// Failed to execute the cmd.
-
-			} else {
-				// DOCKER_PRESERVE mode
-				va_start(arg_lst, container_mode);
-				char container_name[MAX_BUFFER_SIZE];
-				strncpy(container_name, va_arg(arg_lst, const char *), MAX_BUFFER_SIZE - 1);
-				va_end(arg_lst);
-
-				char sub_proc_sh_fn[MAX_BUFFER_SIZE];
-				char sub_proc_sh_fn_path[MAX_BUFFER_SIZE];
-				string_nformat(sub_proc_sh_fn, sizeof(sub_proc_sh_fn), "tmp_%s.sh", p->sandbox);
-				string_nformat(sub_proc_sh_fn_path, sizeof(sub_proc_sh_fn_path), "%s/%s", wrk_space, sub_proc_sh_fn);
-
-				FILE *sub_proc_script_fn = fopen(sub_proc_sh_fn_path, "w");
-				fprintf(sub_proc_script_fn, "%s\ncd %s\n./%s", DEFAULT_EXE_APP, p->sandbox, TMP_SCRIPT);
-				fclose(sub_proc_script_fn);
-				chmod(sub_proc_sh_fn_path, 0755);
-
-				char run_sh_fn[MAX_BUFFER_SIZE];
-				string_nformat(run_sh_fn, sizeof(run_sh_fn), "./%s", sub_proc_sh_fn);
-
-				execl("/usr/bin/docker", "/usr/bin/docker", "exec", container_name, run_sh_fn, (char *) 0);
-				_exit(127);	// Failed to execute the cmd.
-			}
-		}
 	}
 	return 0;
 }
